//TODO: Add commands to turn the monitor on and off

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using SpeechLib;
using StarTrek.Rss;
using System.Drawing;
using System.Net;

namespace StarTrek
{
    /// <summary>
    /// This event is raised when the screen needs to be repainted
    /// </summary>
    /// <param name="sender"></param>
    public delegate void ContentChanged(object sender);

    /// <summary>
    /// The speech class uses the Microsoft Speech SDK to listen for commands.
    /// Multiple grammars are used to keep the possible command set small for 
    /// better precision. The accuracy limits can be adjusted in the settings file.
    /// This class also contains the code for executing the voice commands.
    /// </summary>
    class Speech
    {
        //SAPI
        private SpeechLib.SpSharedRecoContext objRecoContext;
        private SpeechLib.ISpeechRecoGrammar grammar;

        private RssFeed rssFeed;

        string previousGrammar = "StarTrekCommands.xml";	//for safty load something...

        int accuracyLimit = Properties.Settings.Default.AccuracyLimit; // percentage of accuracy required for a match
        float accuracyMax = 0.1F;

        public event ContentChanged ContentChangedEvent;

        private Timer resetMenuTimer = new Timer(); // go back to slideshow if no menu option is selected

        /// <summary>
        /// Create the main object (SpSharedRecoContext), init the gramar
        /// </summary>
        public void initSAPI()
        {
            objRecoContext = new SpeechLib.SpSharedRecoContext();
            objRecoContext.Recognition += new _ISpeechRecoContextEvents_RecognitionEventHandler(RecoContext_Recognition);
            objRecoContext.EventInterests = SpeechLib.SpeechRecoEvents.SRERecognition | SpeechLib.SpeechRecoEvents.SREAudioLevel;

            //create grammar interface with ID = 0
            grammar = objRecoContext.CreateGrammar(0);

            // set up the menu reset timer
            resetMenuTimer.Tick += new EventHandler(ResetMenuTimerProcessor);
            resetMenuTimer.Interval = Properties.Settings.Default.MenuResetTimer * 1000;
        }

        /// <summary>
        /// When this timer fires, reset to the slideshow. This helps to decrease the penalty
        /// for false positives. The screen will reset itself automatically if no further
        /// commands are issued.
        /// </summary>
        /// <param name="myObject"></param>
        /// <param name="myEventArgs"></param>
        private void ResetMenuTimerProcessor(Object myObject, EventArgs myEventArgs)
        {
            Execute("Resume");
        }

        /// <summary>
        /// Loading grammar from file
        /// And resetting all previous rules (delete all previous rules !)
        /// </summary>
        /// <param name="FileName">FileName of speech grammar relative to the application path</param>
        public void SAPIGrammarFromFile(string FileName)
        {
            try
            {
                string appPath = Application.StartupPath + @"\";
                grammar.CmdLoadFromFile(appPath + FileName, SpeechLib.SpeechLoadOption.SLODynamic);
                grammar.CmdSetRuleIdState(0, SpeechRuleState.SGDSActive);

                previousGrammar = FileName;
            }
            catch
            {
                MessageBox.Show("Error loading file " + FileName + "\n", "Error - SAPIGrammarFromFile");
            }
        }

        /// <summary>
        /// Main objRecoContext event
        /// Launched when engine recognized a phrase
        /// If the accuracy bar is achieved, execute the command.
        /// Otherwise (and if debug mode), display what was heard but don't execute.
        /// </summary>
        /// <param name="e">contained information on the phrase that been recognized</param>
        public void RecoContext_Recognition(int StreamNumber, object StreamPosition, SpeechRecognitionType RecognitionType, ISpeechRecoResult e)
        {
            //calculate accuracy
            float accuracy = (float)e.PhraseInfo.Elements.Item(0).EngineConfidence;

            //change accuracyMax dynamicly
            if (accuracyMax < accuracy)
                accuracyMax = accuracy;

            if (accuracy < 0)
                accuracy = 0;

            accuracy = (int)((float)accuracy / accuracyMax * 100);
            string phrase = e.PhraseInfo.GetText(0, -1, true);

            // Set the debug string to the accuracy information
            Display.debugString = String.Format("Accuracy {0}\nPhrase {1}", accuracy.ToString(), phrase == "" ? "" : e.PhraseInfo.Rule.Name);

            if (phrase != "" && accuracy >= accuracyLimit)
            {
                Display.ClearMessage();
                Execute(e.PhraseInfo.Rule.Name);
            }
            else if (ScreenSaverForm.DEBUG)
            {
                 ContentChangedEvent(this);
            }
        }

        /// <summary>
        /// Based on the rule, execute the associated command.
        /// </summary>
        /// <param name="rulename"></param>
        public void Execute(string rulename)
        {
            switch (rulename)
            {
                case "Internet":
                    {
                        //loading Information grammar file
                        resetMenuTimer.Start();
                        SAPIGrammarFromFile("StarTrek_Information.xml");
                        Display.MessageBackground = Properties.Resources.background;
                        Display.MessageTitle = "Internet";
                        Display.MessageBody.Add("Weather\nSteep and Cheap\nSlickDeals\nWoot Daily Deals");
                        ContentChangedEvent(this);
                        break;
                    }
                case "Television":
                    {
                        //loading television grammar file
                        resetMenuTimer.Start();
                        SAPIGrammarFromFile("StarTrek_TV.xml");
                        Display.MessageBackground = Properties.Resources.background;
                        Display.MessageTitle = "Television";
                        Display.MessageBody.Add("ESPN\nESPN2\nFox Sports Net\nSpeed Channel\nABC\nNBC\nCBS\nFox");
                        ContentChangedEvent(this);
                        break;
                    }
                case "Weather":
                    {
                        resetMenuTimer.Stop();
                        Display.MessageBackground = Properties.Resources.background;
                        Display.Waiting();
                        Display.ClearMessageBody();

                        rssFeed = RssFeed.FromUri("http://www.wunderground.com/auto/rss_full/WA/Issaquah.xml");
                        Display.MessageTitle = "Weather for Sammamish, WA";

                        foreach (RssItem item in rssFeed.Channels[0].Items)
                        {
                            string desc = item.Description;
                            desc = desc.Trim();
                            desc = desc.Replace("&#176;", "");
                            desc = desc.Replace(" | ", "\n");
                            int index = desc.IndexOf("<img");
                            if (index >= 0)
                            {
                                desc = desc.Substring(0, index);
                            }

                            Display.MessageBody.Add(desc + "\n");
                        }
                        rssFeed = null;
                        ContentChangedEvent(this);

                        break;
                    }
                case "SlickDeals":
                    {
                        resetMenuTimer.Stop();
                        Display.MessageBackground = Properties.Resources.background;
                        Display.Waiting();
                        Display.ClearMessageBody();

                        rssFeed = RssFeed.FromUri("http://slickdeals.net/rss.php");
                        Display.MessageTitle = rssFeed.Channels[0].Title;

                        int max = (3 < rssFeed.Channels[0].Items.Count) ? 3 : rssFeed.Channels[0].Items.Count;
                        for (int i = 0; i < max; i++)
                        {
                            string desc = rssFeed.Channels[0].Items[i].Title;
                            desc = desc.Trim();

                            Display.MessageBody.Add(desc + "\n");
                        }
                        rssFeed = null;
                        ContentChangedEvent(this);

                        break;
                    }
                case "Woot":
                    {
                        resetMenuTimer.Stop();
                        Display.MessageBackground = Properties.Resources.background;
                        Display.Waiting();
                        Display.ClearMessageBody();

                        rssFeed = RssFeed.FromUri("http://www.woot.com/Blog/Rss.aspx");
                        Display.MessageTitle = rssFeed.Channels[0].Title;

                        RssItem item = rssFeed.Channels[0].Items[0];
                        Display.MessageBody.Add(item.Title);

                        // get the image
                        try
                        {
                            int index1 = item.Description.IndexOf("<img") + 10;
                            int index2 = item.Description.IndexOf("'", index1);
                            string url = item.Description.Substring(index1, index2 - index1);
                            url = url.Replace("thumbnail", "standard");
                            Display.MessageImage = GetWebImage(url);
                        }
                        catch
                        { }

                        rssFeed = null;
                        ContentChangedEvent(this);

                        break;
                    }
                case "SAC":
                    {
                        resetMenuTimer.Stop();
                        Display.MessageBackground = Properties.Resources.background;
                        Display.Waiting();
                        Display.ClearMessageBody();

                        rssFeed = RssFeed.FromUri("http://feeds.feedburner.com/steepandcheap");
                        Display.MessageTitle = rssFeed.Channels[0].Title;

                        RssItem item = rssFeed.Channels[0].Items[0];
                        string desc = item.Description;

                        int index1 = desc.IndexOf("<img") + 10;
                        int index2 = desc.IndexOf("\"", index1);
                        string url = desc.Substring(index1, index2 - index1);
                        Display.MessageImage = GetWebImage(url);


                        index1 = desc.IndexOf("<br") + 6;
                        index2 = desc.IndexOf("<div", index1);
                        desc = desc.Substring(index1, index2 - index1);
                        desc = desc.Replace("<br />", "\n");
                        Display.MessageBody.Add(desc);

                        ContentChangedEvent(this);
                        break;
                    }
                case "Commands":
                    {
                        resetMenuTimer.Stop();
                        Display.MessageBackground = Properties.Resources.background;
                        Display.MessageTitle = "Command List";
                        Display.MessageBody.Add("Weather");
                        Display.MessageBody.Add("Slick Deals");
                        Display.MessageBody.Add("Woot Daily Deals");
                        Display.MessageBody.Add("Steep and Cheap");
                        Display.MessageBody.Add("Resume Slideshow");
                        Display.MessageBody.Add("Televions: ESPN, ESPN2, Fox Sports Net, NBC, CBS, ABC, Fox, Speed Channel");

                        ContentChangedEvent(this);
                        break;
                    }
                case "TV_ESPN":
                    {
                        resetMenuTimer.Stop();
                        StartTVChannel("ESPN", 31);
                        break;
                    }
                case "TV_ESPN2":
                    {
                        resetMenuTimer.Stop();
                        StartTVChannel("ESPN2", 32);
                        break;
                    }
                case "TV_Speed":
                    {
                        resetMenuTimer.Stop();
                        StartTVChannel("Speed Channel", 33);
                        break;
                    }
                case "TV_FSN":
                    {
                        resetMenuTimer.Stop();
                        StartTVChannel("Fox Sports Net", 30);
                        break;
                    }
                case "TV_NBC":
                    {
                        resetMenuTimer.Stop();
                        StartTVChannel("NBC", 5);
                        break;
                    }
                case "TV_CBS":
                    {
                        resetMenuTimer.Stop();
                        StartTVChannel("CBS", 7);
                        break;
                    }
                case "TV_ABC":
                    {
                        resetMenuTimer.Stop();
                        StartTVChannel("ABC", 4);
                        break;
                    }
                case "TV_Fox":
                    {
                        resetMenuTimer.Stop();
                        StartTVChannel("Fox", 13);
                        break;
                    }
                case "Resume":
                    {
                        resetMenuTimer.Stop();
                        SAPIGrammarFromFile("StarTrekCommands.xml");
                        Display.ClearMessage();
                        ContentChangedEvent(this);
                        break;
                    }
            }
        }

        /// <summary>
        /// Download an image from the web.
        /// </summary>
        /// <param name="url"></param>
        /// <returns></returns>
        private Image GetWebImage(string url)
        {
            HttpWebRequest wr = (HttpWebRequest)WebRequest.Create(url);
            HttpWebResponse wresponse = (HttpWebResponse)wr.GetResponse();
            return Image.FromStream(wresponse.GetResponseStream());
        }
        
        /// <summary>
        /// Start the requested TV channel. This will make a call to the server
        /// to start the stream and then activate VLC to display the stream.
        /// </summary>
        /// <param name="channelName"></param>
        /// <param name="channelNumber"></param>
        private void StartTVChannel(string channelName, int channelNumber)
        {
            // The only thing you can say is the resume command
            SAPIGrammarFromFile("StarTrek_Resume.xml");

            Display.MessageBackground = Properties.Resources.background;
            Display.MessageTitle = "TV - " + channelName;
            Display.MessageBody.Add("Starting television...");
            ContentChangedEvent(this);

            try
            {
                Display.StartTVChannel(channelName, channelNumber);
            }
            catch (Exception ex)
            {
                Display.MessageBody.Clear();
                Display.MessageBody.Add(ex.ToString());
                ContentChangedEvent(this);
            }
        }
    }
}
